/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and
*  abiding by the rules of distribution of free software. You can  use,
*  modify and/ or redistribute the software under the terms of the CeCILL-B
*  license as circulated by CEA, CNRS and INRIA at the following URL
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability.
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */

/**
*  \file
*  \brief Class bbtk::vtkGBlackBoxView
*/


#include "vtkGBlackBoxView.h"

namespace bbtk
{


	//=========================================================================
	vtkGBlackBoxView::vtkGBlackBoxView()
	{
	}

	//=========================================================================
	vtkGBlackBoxView::~vtkGBlackBoxView()
	{
	}

	//=========================================================================
    void vtkGBlackBoxView::update_X_Fin()
    {
		double xInic, yInic,zInic,xFin, yFin,zFin;
		_model->getInicPoint(xInic,yInic,zInic);
        double *bounds = _boxTextActor->GetBounds();
		_model->getFinalPoint(xFin, yFin,zFin);

		double xFin1,xFin2,xFin3;
		// position referens from box name
//EED 2020-01-15
//        xFin1 = xInic+(bounds[1]-bounds[0])/2 + 20;
		xFin1 = xInic+(bounds[1]-bounds[0])/10 + 20;
        xFin2=xFin1;
        xFin3=xFin1;
        // position referens from input ports
        GBoxModel* gbm=(GBoxModel*)_model;
		if ( gbm->getNumInputPorts()>0 )
		{
            GPortModel* gpm = gbm->getInputPort( gbm->getNumInputPorts()-1 );
            double xpFin, ypFin, zpFin;
            gpm->getFinalPoint(xpFin, ypFin, zpFin);
            xFin2 = xpFin+1;
        }
        // position referens from output ports
		if ( gbm->getNumOutputPorts()>0 )
		{
            GPortModel* gpm = gbm->getOutputPort( gbm->getNumOutputPorts()-1 );
            double xpFin, ypFin, zpFin;
            gpm->getFinalPoint(xpFin, ypFin, zpFin);
            xFin3 = xpFin+2;
        }
        xFin=xFin1;
        if (xFin2>xFin) xFin=xFin2;
        if (xFin3>xFin) xFin=xFin3;
		_model->setFinalPoint( xFin , yFin , zFin  );
    }

	//=========================================================================
	void vtkGBlackBoxView::update(int idController,int command)
	{
//EED 2017-03)02
		update_X_Fin();
		vtkGBoxView::update(idController,command);
		double xInic, yInic,zInic,xFin, yFin,zFin;
		_model->getInicPoint(xInic,yInic,zInic);
		updatePositionTextActor(xInic, yInic,zInic);
		setRefreshWaiting();
	}

	//=========================================================================

	void vtkGBlackBoxView::createVtkObjects() //virtual
	{

		//------------
		_pts				= vtkPoints::New();
//		vtkCellArray *lines = vtkCellArray::New();
//		vtkPolyData *_pd	= vtkPolyData::New();
//		_borderPolyMapper	= vtkPolyDataMapper::New();
//		_borderObjectActor	= vtkActor::New();

		_pts->SetNumberOfPoints(4);

		double xInic, yInic,zInic,xFin, yFin,zFin;
		_model->getInicPoint(xInic,yInic,zInic);
		_model->getFinalPoint(xFin, yFin,zFin);

		// RaC In the actual version, zInic=zFin=GPOSITION_Z

//EED		_pts->SetPoint(0, xInic, yInic, zInic );
//EED		_pts->SetPoint(1, xInic, yFin, zInic );
//EED		_pts->SetPoint(2, xFin, yFin, zFin );
//EED		_pts->SetPoint(3, xFin, yInic, zFin );

//EED 2017-03-02
//		_pts->SetPoint(0, -1000, -1000, -1000 );
//		_pts->SetPoint(1, xInic, yFin, zInic );
//		_pts->SetPoint(2, 1000, 1000, 1000 );
//		_pts->SetPoint(3, xFin, yInic, zFin );


//		lines->InsertNextCell(5);
//		lines->InsertCellPoint(0);
//		lines->InsertCellPoint(1);
//		lines->InsertCellPoint(2);
//		lines->InsertCellPoint(3);
//		lines->InsertCellPoint(0);

//		_pd->SetPoints( _pts );
//		_pd->SetLines( lines );
//		_borderPolyMapper->SetInput(_pd);
//		_borderObjectActor->SetMapper(_borderPolyMapper);
//		_borderPolyMapper->Modified();


		///************************* FILL *************************
		vtkCellArray *strip = vtkCellArray::New();
		vtkPolyData *pdFill = vtkPolyData::New();
		_fillPolyMapper		= vtkPolyDataMapper::New();
		_fillObjectActor	= vtkActor::New();
		// RaC In the actual version, zInic=zFin=GPOSITION_Z
		strip->InsertNextCell(5);
		strip->InsertCellPoint(0);
		strip->InsertCellPoint(1);
		strip->InsertCellPoint(2);
		strip->InsertCellPoint(0);
		strip->InsertCellPoint(3);
		pdFill->SetPoints( _pts );
		pdFill->SetStrips( strip );


//EED 2017-01-01 Migration VTK7
#if VTK_MAJOR_VERSION <= 5
		_fillPolyMapper->SetInput(pdFill);
#else
		_fillPolyMapper->SetInputData(pdFill);
#endif


		_fillObjectActor->SetMapper(_fillPolyMapper);
		_fillPolyMapper->Modified();
		///************************* FILL *************************



		createVtkInputOutputPorts();



		//------------

//        _boxTextActor            = vtkTextActor::New();  // for actor 2D
		_boxTextActor			= vtkTextActor3D::New();
		vtkTextProperty *prop 	=  vtkTextProperty::New();
		prop->BoldOn();
		prop->SetFontFamilyToArial();
        prop->ShadowOff();
//EED 2020-01-15
//		prop->SetFontSize(80);
//        prop->SetFontSize(15);
        prop->SetFontSize(40);
		_boxTextActor->SetTextProperty(prop);
		//------------

		updatePositionTextActor( xInic,  yInic,  zInic);
	}

	//===============BoundaryEdgesOn==========================================================

	void vtkGBlackBoxView::addVtkActors()//virtual
	{
		
//		_baseView->GetRenderer()->AddActor(_borderObjectActor);
		vtkGBoxView::addVtkActors();
		_baseView->GetRenderer()->AddActor(_boxTextActor);

	}

	//=========================================================================

	void vtkGBlackBoxView::removeVtkActors()//virtual
	{
		_baseView->GetRenderer()->RemoveActor(_boxTextActor);
//		_baseView->GetRenderer()->RemoveActor(_borderObjectActor);
		vtkGBoxView::removeVtkActors();
	}

	//=========================================================================

	void vtkGBlackBoxView::updatePositionTextActor(double xInic, double yInic, double zInic)
	{
		std::string temp	= _model->getBBTKType();
		temp+=":";
		temp+=_model->getBBTKName();
		_boxTextActor->SetInput(temp.c_str());
//        _boxTextActor->SetPosition(xInic+4,yInic-7);  // for actor 2D
//EED 2020-01-15
//		_boxTextActor->SetPosition(xInic+4,yInic-7,zInic+1);
        _boxTextActor->SetPosition(xInic+1.1,yInic-BOX_HEIGHT,zInic+1);
		_boxTextActor->SetScale(0.05,0.05,1);
	}

	//=========================================================================

	void vtkGBlackBoxView::updateColors()
	{
		_fillObjectActor->GetProperty()->SetAmbient(0.6);

		if(_state==NOTHING_HAPPENS)
		{
//			_borderObjectActor->GetProperty()->SetLineWidth(1);
///			_borderObjectActor->GetProperty()->SetColor(BOXBORDER_NH_R,BOXBORDER_NH_G,BOXBORDER_NH_B);
			_fillObjectActor->GetProperty()->SetColor(BOXFILL_NH_R,BOXFILL_NH_G,BOXFILL_NH_B);
			_boxTextActor->GetTextProperty()->SetColor(BOXTEXT_NH_R,BOXTEXT_NH_G,BOXTEXT_NH_B);

			GBlackBoxModel *bbmodel = (GBlackBoxModel*)_model;
			if(bbmodel->isExecutable())
			{
//				_borderObjectActor->GetProperty()->SetColor(BOXBORDER_NH_R,0.9,BOXBORDER_HIGHLIGHTED_B);
				_fillObjectActor->GetProperty()->SetColor(BOXFILL_NH_R,0.9,BOXFILL_HIGHLIGHTED_B);
			}

		}
		else if(_state==HIGHLIGHTED)
		{
//			_borderObjectActor->GetProperty()->SetLineWidth(2);
//			_borderObjectActor->GetProperty()->SetColor(BOXBORDER_HIGHLIGHTED_R,BOXBORDER_HIGHLIGHTED_G,BOXBORDER_HIGHLIGHTED_B);
			_fillObjectActor->GetProperty()->SetColor(BOXFILL_HIGHLIGHTED_R,BOXFILL_HIGHLIGHTED_G,BOXFILL_HIGHLIGHTED_B);
			_boxTextActor->GetTextProperty()->SetColor(BOXTEXT_HIGHLIGHTED_R,BOXTEXT_HIGHLIGHTED_G,BOXTEXT_HIGHLIGHTED_B);
		}
		else if(_state==DRAG)
		{
//			_borderObjectActor->GetProperty()->SetLineWidth(1.2);
//			_borderObjectActor->GetProperty()->SetColor(BOXBORDER_DRAG_R,BOXBORDER_DRAG_G,BOXBORDER_DRAG_B);
			_fillObjectActor->GetProperty()->SetColor(BOXFILL_DRAG_R,BOXFILL_DRAG_G,BOXFILL_DRAG_B);
			_boxTextActor->GetTextProperty()->SetColor(BOXTEXT_DRAG_R,BOXTEXT_DRAG_G,BOXTEXT_DRAG_B);
		}
		else if(_state==SELECTED)
		{
//			_borderObjectActor->GetProperty()->SetLineWidth(2);
//			_borderObjectActor->GetProperty()->SetColor(BOXBORDER_SELECTED_R,BOXBORDER_SELECTED_G,BOXBORDER_SELECTED_B);
			_fillObjectActor->GetProperty()->SetColor(BOXFILL_SELECTED_R,BOXFILL_SELECTED_G,BOXFILL_SELECTED_B);
			_boxTextActor->GetTextProperty()->SetColor(BOXTEXT_SELECTED_R,BOXTEXT_SELECTED_G,BOXTEXT_SELECTED_B);
		}

		updateColorsPorts();

	}

	//=========================================================================


}  // EO namespace bbtk

// EOF

